@@ -106,7 +106,7 @@ def generate_checksums_file_content(
 def verify_checksum(entry: Tuple[str, str]) -> Optional[str]:
     """Check whether the checksum of a given file matches the specified checksum.
 
-    :param record: tuple of (checksum value, file to verify path)
+    :param entry: tuple of (checksum value, file to verify path)
     :return: None if the checksums match, or the file's actual checksum if
         it does not match the specified checksum.
     """
@@ -158,7 +158,7 @@ def read_checksum_file(checksum_file: IO[bytes]) -> Iterable[Tuple[str, str]]:
     that the function is more flexible and also allows to read input from
     a stream rather than an actual file.
 
-    :param f: file object.
+    :param checksum_file: file object.
     :return: generator of tuples of the form (checksum value, file path).
     :raises UserError:
     """
