@@ -2,29 +2,29 @@ import enum
 import io
 import re
 import warnings
+from functools import partial, wraps
 from typing import (
+    IO,
+    Any,
+    Callable,
     Iterable,
     List,
+    Optional,
     Sequence,
     Tuple,
-    Optional,
-    Union,
     Type,
-    IO,
-    Callable,
-    cast,
-    Any,
     TypeVar,
+    Union,
+    cast,
 )
-from functools import wraps, partial
 
 from libbiomedit import crypt
+
+from ..utils.config import Config
 from . import gpg
 from .error import UserError
 from .request import urlopen
 from .secret import Secret, reveal
-from ..utils.config import Config
-
 
 GPGStore = gpg.GPGStore
 ExceptionType = Union[Type[BaseException], Tuple[Type[BaseException], ...]]
@@ -104,10 +104,40 @@ def create_revocation_certificate(
     return gpg_store.gen_revoke(fingerprint, passphrase=passphrase)
 
 
+class KeyAlgorithm(enum.Enum):
+    """Encryption algorithm used in a public PGP key.
+    The numeric values associated to algorithms follow the GnuPG settings.
+    """
+
+    RSA = 1
+    DSA = 2
+    ECC = 22
+
+
 def verify_key_length(key: gpg.Key, min_key_length: int = 4096) -> None:
+    try:
+        algorithm = KeyAlgorithm(key.pub_key_algorithm)
+    except ValueError as exc:
+        if key.key_length < min_key_length:
+            raise UserError(
+                f"Not allowed: key {key.key_id} is using an algorithm which we do not allow. "
+                f"Use either the RSA algorithm (min. key length: {min_key_length}) "
+                "or the ECC algorithm instead."
+            ) from exc
+    if algorithm == KeyAlgorithm.ECC:
+        msg = (
+            f"Please be aware: key {key.key_id} is of type ECC, which is not supported "
+            "by older versions of GnuPG, available on older operating systems."
+        )
+        warnings.warn(msg)
+        return
+    if algorithm == KeyAlgorithm.DSA:
+        raise UserError(
+            f"Not allowed: key {key.key_id} is of type DSA, which we do not allow."
+        )
     if key.key_length < min_key_length:
         raise UserError(
-            f"Key {key.key_id} is shorter than the minimal "
+            f"Not allowed: key {key.key_id} is shorter than the minimal "
             f"required length of {min_key_length} bit"
         )
 
