@@ -13,12 +13,13 @@ from typing import (
     Type,
 )
 
+from .theme import Action, IconRepainterWidget, PushButton
 from .model import AppData
 from .parallel import run_thread
-from .pyside import QtCore, QtWidgets, QAction, QtGui, open_window
+from .pyside import QtCore, QtWidgets, open_window
 from .. import APP_NAME_SHORT
-from ..utils.progress import ProgressInterface
 from ..core.secret import Secret
+from ..utils.progress import ProgressInterface
 
 EXTRA_HEIGHT = 1
 MACOS_TOOLTIP_BG_COLOR = "#ffffca"
@@ -80,7 +81,7 @@ class SelectionButton(QtWidgets.QPushButton):
         self.setEnabled(bool(len(self._selection_model.selectedRows())))
 
 
-class SelectionAction(QAction):
+class SelectionAction(Action):
     """An action extension which connects this action to a selection model.
 
     The action is disabled by default. And gets enabled when the selection has,
@@ -119,14 +120,12 @@ class GuiProgress(QtCore.QObject, ProgressInterface):
         return self.n
 
 
-class ConsoleWidget(QtWidgets.QGroupBox):
+class ConsoleWidget(QtWidgets.QGroupBox, IconRepainterWidget):
     def __init__(self, title: str, parent: QtWidgets.QWidget):
         super().__init__(title, parent)
         self.textbox = QtWidgets.QTextEdit()
         self.textbox.setReadOnly(True)
-        btn_clear_console = QtWidgets.QPushButton(
-            QtGui.QIcon(":icon/feather/slash.png"), ""
-        )
+        btn_clear_console = PushButton(":icon/feather/slash.png", "", self)
         btn_clear_console.setToolTip("Clear console")
         btn_clear_console.clicked.connect(self.clear)  # type: ignore
         layout = QtWidgets.QHBoxLayout(self)
@@ -145,35 +144,29 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         self.textbox.append(text)
 
 
-class PathInput(QtWidgets.QWidget):
-    """Path selection widget with a select button and a 'show path' field."""
+class PathInput:
+    """Path selection container with a select button and a 'show path' field."""
 
     def __init__(
         self,
-        *args: Any,
+        parent: IconRepainterWidget,
         directory: bool = True,
         path: Optional[Path] = Path.home(),
-        **kwargs: Any,
+        btn_clear: bool = True,
     ):
-        super().__init__(*args, **kwargs)
-        # Note: cast() is used here because self.parent() wrongly hints that
-        # it returns a QtCore.QObject object.
-        self.parent_widget = cast(QtWidgets.QWidget, self.parent())
-        self.text = LineEdit(self.parent_widget)
+        self.parent = parent
+        self.text = LineEdit(parent)
         self.text.setReadOnly(True)
-        self.btn = QtWidgets.QPushButton(
-            QtGui.QIcon(f":icon/feather/{'folder' if directory else 'file'}.png"),
-            "",
+        self.btn = PushButton(
+            f":icon/feather/{'folder' if directory else 'file'}.png", "", parent
         )
         self.btn.setToolTip("Change location")
         self.btn.clicked.connect(partial(self._update_location, directory))  # type: ignore
-
-        # Add additional button to allow clearing the selected path.
-        self.btn_clear = QtWidgets.QPushButton(
-            QtGui.QIcon(":icon/feather/slash.png"), ""
-        )
-        self.btn_clear.setToolTip("Clear location")
-        self.btn_clear.clicked.connect(self._clear_location)  # type: ignore
+        if btn_clear:
+            # Add additional button to allow clearing the selected path.
+            self.btn_clear = PushButton(":icon/feather/slash.png", "", parent)
+            self.btn_clear.setToolTip("Clear location")
+            self.btn_clear.clicked.connect(self._clear_location)  # type: ignore
         self.update_path(path)
 
     def update_path(self, path: Optional[Path]) -> None:
@@ -188,11 +181,13 @@ class PathInput(QtWidgets.QWidget):
             location = Path.home()
         if directory:
             new_path = QtWidgets.QFileDialog.getExistingDirectory(
-                parent=self.parent_widget, caption="Select Directory", dir=str(location)
+                parent=self.parent,
+                caption="Select Directory",
+                dir=str(location),
             )
         else:
             new_path, _ = QtWidgets.QFileDialog.getOpenFileName(
-                parent=self.parent_widget, caption="Select File", dir=str(location)
+                parent=self.parent, caption="Select File", dir=str(location)
             )
         if new_path:
             self.update_path(Path(new_path))
@@ -208,20 +203,18 @@ class PathInput(QtWidgets.QWidget):
         self.text.editingFinished.connect(lambda: fn(self.path))  # type: ignore
 
 
-class TabMixin(QtWidgets.QWidget):
+class BaseTab(IconRepainterWidget):
     """Adds two boxes to the layout of a QWidget (a Tab from the application):
     - Box with buttons to run and test run a workflow.
     - Box with a console.
-
-    This class is a "mixin" and is destined to be inherited by another class
-    that will provide the constructor.
     """
 
     def create_console(self) -> None:
+        # pylint:disable=attribute-defined-outside-init
         self.console = ConsoleWidget(title="Console", parent=self)
 
     def create_progress_bar(self) -> None:
-        # NOTE: same problem here as above.
+        # pylint:disable=attribute-defined-outside-init
         self.progress_bar = QtWidgets.QProgressBar(self)
 
     @staticmethod
@@ -237,14 +230,17 @@ class TabMixin(QtWidgets.QWidget):
     def create_run_panel(
         self, panel_name: str, action: Callable[..., Any], action_name: str
     ) -> None:
+        # pylint:disable=attribute-defined-outside-init
         self.run_panel = QtWidgets.QGroupBox(panel_name)
-        self.btn_test = TabMixin._create_disabled_button("Test")
+        # pylint:disable=attribute-defined-outside-init
+        self.btn_test = BaseTab._create_disabled_button("Test")
 
         # On pressed button, make sure that the focus switches to that
         # button (Mac specific issue)
         self.btn_test.pressed.connect(self.btn_test.setFocus)  # type: ignore
         self.btn_test.clicked.connect(partial(action, dry_run=True))  # type: ignore
-        self.btn_run = TabMixin._create_disabled_button(action_name)
+        # pylint:disable=attribute-defined-outside-init
+        self.btn_run = BaseTab._create_disabled_button(action_name)
 
         # On pressed button, make sure that the focus switches to that
         # button (Mac specific issue)
