@@ -2,7 +2,7 @@ from functools import partial, wraps
 from typing import Any, cast
 
 from .component import (
-    TabMixin,
+    BaseTab,
     PathInput,
     GuiProgress,
     create_slider,
@@ -17,15 +17,15 @@ from .component import (
     hbox_layout,
 )
 from .file_selection_widget import DirectoryAndFileSelectionWidget
+from .theme import Icon, PushButton
 from .model import TableModel, AppData, KeyValueListModel
 from .pyside import QtCore, QtGui, QtWidgets
-from ..utils.config import LABEL_COMPRESSION_LEVEL, DESCRIPTION_COMPRESSION_LEVEL
-from ..core.metadata import Purpose
 from ..core.filesystem import OutOfSpaceError
+from ..core.metadata import Purpose
+from ..utils.config import LABEL_COMPRESSION_LEVEL, DESCRIPTION_COMPRESSION_LEVEL
 from ..utils.validation import PACKAGE_SUFFIX
 from ..workflows import encrypt
 
-
 ignore_disk_space_error_label = "Ignore disk space error"
 
 
@@ -40,7 +40,7 @@ def encrypt_workflow(*args: str, **kwargs: Any) -> None:
         ) from e
 
 
-class EncryptTab(TabMixin):
+class EncryptTab(BaseTab):
     """Class that creates the "Encrypt" Tab of the GUI application."""
 
     def __init__(self, parent: QtWidgets.QMainWindow, app_data: AppData):
@@ -100,14 +100,16 @@ class EncryptTab(TabMixin):
             QtWidgets.QTableView.SelectionMode.SingleSelection
         )
 
-        recipients_btn_add = QtWidgets.QPushButton(
-            QtGui.QIcon(":icon/feather/user-plus.png"), ""
-        )
+        recipients_btn_add = PushButton(":icon/feather/user-plus.png", "", self)
         recipients_btn_add.setToolTip("Add recipient to the list")
         recipients_btn_remove = SelectionButton(
             "", recipients_output_view.selectionModel()
         )
-        recipients_btn_remove.setIcon(QtGui.QIcon(":icon/feather/user-minus.png"))
+        icon_file_name = ":icon/feather/user-minus.png"
+        recipients_btn_remove.setIcon(Icon(icon_file_name))
+        self.icon_repainter().register(
+            lambda: recipients_btn_remove.setIcon(Icon(icon_file_name))
+        )
         recipients_btn_remove.setToolTip("Remove recipient from the list")
 
         def update_sender(index: int) -> None:
@@ -190,7 +192,9 @@ class EncryptTab(TabMixin):
         output_suffix.setValidator(
             QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(PACKAGE_SUFFIX))
         )
-        output_location = PathInput(path=self.app_data.encrypt_output_location)
+        output_location = PathInput(
+            path=self.app_data.encrypt_output_location, parent=self, btn_clear=False
+        )
         output_location.setStatusTip("Destination folder for the encrypted package")
 
         # Add actions
