@@ -9,10 +9,11 @@ from .component import (
     ToolBar,
     vbox_layout,
 )
+from .theme import Action, IconRepainterWidget
 from .pyside import QtWidgets, QtCore, QtGui, QAction, open_window
 
 
-class FileSelectionWidget(QtWidgets.QGroupBox):
+class FileSelectionWidget(QtWidgets.QGroupBox, IconRepainterWidget):
     """File selection widget."""
 
     def __init__(
@@ -69,7 +70,7 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
         ]
 
     def _create_add_files_action(self) -> QAction:
-        action = QAction(QtGui.QIcon(":icon/feather/file-plus.png"), "Add file", self)
+        action = Action(":icon/feather/file-plus.png", "Add file", self)
         action.triggered.connect(self._add_files)  # type: ignore
         return action
 
@@ -80,7 +81,7 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
             self.file_list_model.layoutChanged.emit()  # type: ignore
 
         action = SelectionAction(
-            QtGui.QIcon(":icon/feather/file-minus.png"),
+            ":icon/feather/file-minus.png",
             "Remove selected",
             self,
             selection_model=self.file_list_view.selectionModel(),
@@ -95,7 +96,7 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
             self.file_list_model.setStringList([])
             self.file_list_model.layoutChanged.emit()  # type: ignore
 
-        action = QAction(QtGui.QIcon(":icon/feather/trash-2.png"), "Clear list", self)
+        action = Action(":icon/feather/trash-2.png", "Clear list", self)
         action.triggered.connect(clear_list)  # type: ignore
         return action
 
@@ -140,9 +141,7 @@ class DirectoryAndFileSelectionWidget(FileSelectionWidget):
             )
             self._update_paths((directory,))
 
-        action = QAction(
-            QtGui.QIcon(":icon/feather/folder-plus.png"), "Add directory", self
-        )
+        action = Action(":icon/feather/folder-plus.png", "Add directory", self)
         action.triggered.connect(add_directory)  # type: ignore
         return action
 
