@@ -1,53 +1,50 @@
 import html
+import logging
+import warnings
 from pathlib import Path
 from typing import (
-    Iterable,
+    Any,
+    Callable,
     Iterator,
     Optional,
     Sequence,
     Tuple,
-    Callable,
-    Any,
     cast,
 )
 
 from sett.core.error import UserError
 from .component import (
+    GridLayoutCell,
+    LineEdit,
     SelectionAction,
     SelectionButton,
     ToolBar,
-    LineEdit,
-    warning_callback,
-    show_warning,
     grid_layout,
-    GridLayoutCell,
+    show_warning,
+    warning_callback,
 )
-from .model import AppData, KeyValueListModel
+from .theme import Action, IconRepainterWidget
 from .keys_refresh import load_authority_key_threaded
+from .model import AppData, KeyValueListModel
 from .parallel import run_thread
-from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
+from .pyside import QAction, QtCore, QtGui, QtWidgets, open_window
 from ..core import crypt, gpg
 from ..core.secret import Secret
 from ..utils.config import Config
-from ..workflows import (
-    upload_keys as upload_keys_workflow,
-    request_sigs as request_sigs_workflow,
-)
+from ..workflows import request_sigs as request_sigs_workflow
+from ..workflows import upload_keys as upload_keys_workflow
 
 
-class KeysTab(QtWidgets.QWidget):
+class KeysTab(IconRepainterWidget):
     def __init__(self, parent: QtWidgets.QMainWindow, app_data: AppData):
         super().__init__(parent=parent)
         self.app_data = app_data
 
-        # Update public and private key lists in AppData.
-        self.update_private_keys()
-        self.update_public_keys()
-
-        # At listeners that refresh the 'Keys' tab of the GUI to reflect
+        # Add a listener which refreshes `Keys` tab to reflect
         # changes in validation authority key.
-        for callback in (self.update_public_keys, self.update_display_selected_pub_key):
-            self.app_data.add_listener("validation_authority_key", callback)
+        self.app_data.add_listener(
+            "validation_authority_key", self.update_display_selected_pub_key
+        )
 
         # Download/refresh the validation authority key.
         load_authority_key_threaded(self.app_data)
@@ -77,21 +74,21 @@ class KeysTab(QtWidgets.QWidget):
             lambda: self.priv_keys_view.selectionModel().clear()
         )
 
-        action_generate_key = QAction(
-            QtGui.QIcon(":icon/feather/plus-square.png"),
+        action_generate_key = Action(
+            ":icon/feather/plus-square.png",
             "Generate new private/public key",
             self,
         )
         action_generate_key.triggered.connect(lambda: KeyGenDialog(parent=self).show())  # type: ignore
-        action_refresh_keys = QAction(
-            QtGui.QIcon(":icon/feather/refresh-cw.png"),
+        action_refresh_keys = Action(
+            ":icon/feather/refresh-cw.png",
             "Refresh keys from the local keyring",
             self,
         )
 
         def refresh_keys() -> None:
-            self.update_private_keys()
-            self.update_public_keys()
+            self.app_data.update_private_keys()
+            self.app_data.update_public_keys()
             self.update_display_selected_pub_key()
 
         action_refresh_keys.triggered.connect(refresh_keys)  # type: ignore
@@ -188,8 +185,8 @@ class KeysTab(QtWidgets.QWidget):
 
             icon_obj = f":icon/feather/{icon}.png"
             if self.app_data.config.offline:
-                action = QAction(
-                    QtGui.QIcon(icon_obj),
+                action = Action(
+                    icon_obj,
                     f"{tip} (not available in offline mode)",
                     self,
                 )
@@ -197,13 +194,13 @@ class KeysTab(QtWidgets.QWidget):
             else:
                 if selection:
                     action = SelectionAction(
-                        QtGui.QIcon(icon_obj),
+                        icon_obj,
                         tip,
                         self,
                         selection_model=selection_model,
                     )
                 else:
-                    action = QAction(QtGui.QIcon(icon_obj), tip, self)
+                    action = Action(icon_obj, tip, self)
             return action
 
         # Create a dict of actions (functions) to associate to each
@@ -234,7 +231,7 @@ class KeysTab(QtWidgets.QWidget):
             ),
             (
                 SelectionAction(
-                    QtGui.QIcon(":icon/feather/trash-2.png"),
+                    ":icon/feather/trash-2.png",
                     "Delete selected keys from your computer",
                     self,
                     selection_model=selection_model,
@@ -242,8 +239,8 @@ class KeysTab(QtWidgets.QWidget):
                 self.delete_keys,
             ),
             (
-                QAction(
-                    QtGui.QIcon(":icon/feather/file-plus.png"),
+                Action(
+                    ":icon/feather/file-plus.png",
                     "Import key from file",
                     self,
                 ),
@@ -254,31 +251,6 @@ class KeysTab(QtWidgets.QWidget):
             action.triggered.connect(fn)  # type: ignore
             yield action
 
-    def update_private_keys(self) -> None:
-        """Retrieve/reload all private keys from the user's local keyring and
-        store them in the application's memory."""
-        keys_private = self.app_data.config.gpg_store.list_sec_keys()
-        try:
-            default_key = (
-                self.app_data.config.default_sender
-                or self.app_data.config.gpg_store.default_key()
-            )
-            self.app_data.default_key_index = next(
-                index
-                for index, entry in enumerate(keys_private)
-                if entry.fingerprint == default_key
-            )
-            self.app_data.encrypt_sender = default_key
-        except StopIteration:
-            pass
-        self.app_data.priv_keys_model.set_data(data=keys_as_tuple(keys=keys_private))
-
-    def update_public_keys(self) -> None:
-        """Retrieve/reload all public keys from the user's local keyring and
-        store them in the application's memory."""
-        keys_public = self.app_data.config.gpg_store.list_pub_keys(sigs=True)
-        self.app_data.pub_keys_model.set_data(data=keys_as_tuple(keys=keys_public))
-
     def get_selected_keys(self) -> Tuple[gpg.Key, ...]:
         """Returns the gpg.Key objects corresponding to the keys currently
         selected in the GUI.
@@ -297,7 +269,7 @@ class KeysTab(QtWidgets.QWidget):
         if keys_to_update:
 
             def on_result() -> None:
-                self.update_public_keys()
+                self.app_data.update_public_keys()
                 self.update_display_selected_pub_key()
                 show_ok()
 
@@ -325,7 +297,7 @@ class KeysTab(QtWidgets.QWidget):
                 with open(path, encoding="utf-8") as fin:
                     key_data = fin.read()
                 crypt.import_keys(key_data, self.app_data.config.gpg_store)
-                self.update_public_keys()
+                self.app_data.update_public_keys()
                 self.update_display_selected_pub_key()
                 msg.setIcon(QtWidgets.QMessageBox.Information)
                 msg.setText("Key has been imported.")
@@ -376,7 +348,7 @@ class KeysTab(QtWidgets.QWidget):
                     msg_warn.setText(format(e))
                     open_window(msg_warn)
                 self.text_panel.clear()
-        self.update_public_keys()
+        self.app_data.update_public_keys()
 
     def upload_key(self) -> None:
         """Upload the selected keys to the keyserver specific in the user's
@@ -394,7 +366,21 @@ class KeysTab(QtWidgets.QWidget):
         keys_to_upload = self.get_selected_keys()
         for key in keys_to_upload:
             msg.setDetailedText(self.key_to_text(key))
+
             if key is keys_to_upload[0]:
+                # check keylength
+                try:
+                    with warnings.catch_warnings(record=True) as w:
+                        crypt.verify_key_length(key)
+                        if len(w) > 0:
+                            logging.warning(w[-1].message)
+                            msg.setIcon(QtWidgets.QMessageBox.Warning)
+                            msg.setText(msg.text() + "\n" + str(w[-1].message))
+                except UserError as exc:
+                    logging.error(str(exc))
+                    msg.setIcon(QtWidgets.QMessageBox.Critical)
+                    msg.setText(str(exc))
+                    msg.setStandardButtons(QtWidgets.QMessageBox.Cancel)
                 click_show_details(msgbox=msg)
             status = open_window(msg)
             if status == QtWidgets.QMessageBox.Ok:
@@ -403,10 +389,12 @@ class KeysTab(QtWidgets.QWidget):
                     f_kwargs=dict(
                         key_ids=[key.fingerprint], config=self.app_data.config
                     ),
-                    report_config=self.app_data.config,
                     capture_loggers=(upload_keys_workflow.logger,),
+                    report_config=self.app_data.config,
                     forward_errors=warning_callback("GPG key upload error"),
-                    signals=dict(result=lambda _: show_ok()),
+                    signals=dict(
+                        result=lambda _: show_ok(),
+                    ),
                 )
 
     def send_signature_request(self) -> None:
@@ -525,7 +513,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
         if keys_to_download:
 
             def on_result() -> None:
-                self.parent_tab.update_public_keys()
+                self.parent_tab.app_data.update_public_keys()
                 self.parent_tab.update_display_selected_pub_key()
 
             self.btn_download.setEnabled(False)
@@ -646,8 +634,8 @@ class KeyGenDialog(QtWidgets.QDialog):
             ):
                 request_signature((key,), self.parent_tab.app_data.config, self)
             self.clear_form()
-            self.parent_tab.update_private_keys()
-            self.parent_tab.update_public_keys()
+            self.parent_tab.app_data.update_private_keys()
+            self.parent_tab.app_data.update_public_keys()
             self.close()
 
     def create_private_key(self) -> None:
@@ -712,6 +700,20 @@ def request_signature(
     for key in keys:
         msg.setDetailedText(KeysTab.key_to_text(key))
         if key is keys[0]:
+            # check keylength and display warnings or errors
+            try:
+                with warnings.catch_warnings(record=True) as w:
+                    crypt.verify_key_length(key)
+                    if len(w) > 0:
+                        logging.warning(w[-1].message)
+                        msg.setIcon(QtWidgets.QMessageBox.Warning)
+                        msg.setText(msg.text() + "\n" + str(w[-1].message))
+            except UserError as exc:
+                logging.error(str(exc))
+                msg.setIcon(QtWidgets.QMessageBox.Critical)
+                msg.setText(str(exc))
+                msg.setStandardButtons(QtWidgets.QMessageBox.Cancel)
+
             click_show_details(msgbox=msg)
         status = open_window(msg)
         if status == QtWidgets.QMessageBox.Yes:
@@ -736,8 +738,3 @@ def ok_message(
     msg_ok.setWindowTitle(title)
     msg_ok.setText(msg)
     return lambda: open_window(msg_ok)
-
-
-def keys_as_tuple(keys: Sequence[gpg.Key]) -> Iterable[Tuple[str, gpg.Key]]:
-    """Returns a generator of tuples of the form ("userID keyID", key)."""
-    return ((f"{key.uids[0]} {key.key_id}", key) for key in keys)
