@@ -185,6 +185,30 @@ class AppData(ClassWithListener):
 
     validation_authority_key: Optional[gpg.Key] = None
 
+    def update_private_keys(self) -> None:
+        """Retrieve/reload all private keys from the user's local keyring and
+        store them in the application's memory."""
+        keys_private = self.config.gpg_store.list_sec_keys()
+        try:
+            default_key = (
+                self.config.default_sender or self.config.gpg_store.default_key()
+            )
+            self.default_key_index = next(
+                index
+                for index, entry in enumerate(keys_private)
+                if entry.fingerprint == default_key
+            )
+            self.encrypt_sender = default_key
+        except StopIteration:
+            pass
+        self.priv_keys_model.set_data(data=keys_as_tuple(keys=keys_private))
+
+    def update_public_keys(self) -> None:
+        """Retrieve/reload all public keys from the user's local keyring and
+        store them in the application's memory."""
+        keys_public = self.config.gpg_store.list_pub_keys(sigs=True)
+        self.pub_keys_model.set_data(data=keys_as_tuple(keys=keys_public))
+
     def __post_init__(self) -> None:
         super().__init__()
         if self.config.offline:
@@ -197,3 +221,11 @@ class AppData(ClassWithListener):
         if self.config.package_name_suffix is not None:
             self.encrypt_package_name_suffix = self.config.package_name_suffix
         self.transfer_verify_package_name = self.config.verify_package_name
+        self.update_private_keys()
+        self.update_public_keys()
+        self.add_listener("validation_authority_key", self.update_public_keys)
+
+
+def keys_as_tuple(keys: Sequence[gpg.Key]) -> Iterable[Tuple[str, gpg.Key]]:
+    """Returns a generator of tuples of the form ("userID keyID", key)."""
+    return ((f"{key.uids[0]} {key.key_id}", key) for key in keys)
