@@ -1,14 +1,8 @@
-from typing import Optional, Tuple, Union, Dict, Callable, Sequence, Any, cast
+import re
 from dataclasses import Field, asdict, fields  # pylint: disable=unused-import
+from typing import Optional, Tuple, Union, Dict, Callable, Sequence, Any, cast
 
-from .listener import Listener
-from .ui_model_bind import (
-    bind,
-    TextControl,
-    PathControl,
-    BoolControl,
-    NumericControl,
-)
+from sett.utils.validation import REGEX_FQDN, REGEX_IP
 from .component import (
     PathInput,
     create_slider,
@@ -17,9 +11,18 @@ from .component import (
     grid_layout,
     GridLayoutCell,
 )
+from .theme import Icon, IconRepainterWidget, PushButton
+from .keys_refresh import load_authority_key_threaded
+from .listener import Listener
 from .model import AppData
 from .pyside import QtCore, QtGui, QtWidgets
-from .keys_refresh import load_authority_key_threaded
+from .ui_model_bind import (
+    bind,
+    TextControl,
+    PathControl,
+    BoolControl,
+    NumericControl,
+)
 from ..utils import config
 from ..utils.config import Config, FileType
 
@@ -66,8 +69,9 @@ class ConfigProxy(Listener):
         return cfg
 
 
-class SettingsTab(QtWidgets.QWidget):
+class SettingsTab(IconRepainterWidget):
     persist_btn_text = "Save to config file"
+    persist_btn_icon_file_name = ":icon/feather/save.png"
 
     def __init__(self, parent: QtWidgets.QMainWindow, app_data: AppData):
         super().__init__(parent=parent)
@@ -79,7 +83,10 @@ class SettingsTab(QtWidgets.QWidget):
 
         def cfg_field(key: str) -> Tuple[QtWidgets.QWidget, ...]:
             return widget_row_from_field(
-                self.config_proxy, cfg_fields[key], widget_register=widget_register
+                self.config_proxy,
+                cfg_fields[key],
+                parent=parent,
+                widget_register=widget_register,
             )
 
         self.setLayout(
@@ -108,6 +115,7 @@ class SettingsTab(QtWidgets.QWidget):
                     cfg_field("ssh_password_encoding"),
                     cfg_field("dcc_portal_url"),
                     cfg_field("verify_package_name"),
+                    cfg_field("sftp_buffer_size"),
                 ),
                 group_box(
                     "sett updates", cfg_field("repo_url"), cfg_field("check_version")
@@ -135,6 +143,7 @@ class SettingsTab(QtWidgets.QWidget):
             self._enable_checkbox_key_autodownload
         )
         self.config_proxy.refresh()
+        self.icon_repainter().register(self.refresh_icon)
 
     def _save_config_to_disk(self) -> None:
         """Write the current in-app config values to a config file on disk."""
@@ -163,17 +172,21 @@ class SettingsTab(QtWidgets.QWidget):
         return (GridLayoutCell(top_label, span=2), reset_btn, defaults_btn)
 
     def _footer(self) -> Tuple[GridLayoutCell]:
-        persist_btn = QtWidgets.QPushButton(self.persist_btn_text, self)
-        persist_btn.setIcon(QtGui.QIcon(":icon/feather/save.png"))
-        persist_btn.setStatusTip("Saves your current settings to the config file")
-        persist_btn.clicked.connect(self._save_config_to_disk)  # type: ignore
-        return (GridLayoutCell(persist_btn, span=4),)
+        self.persist_btn = PushButton(
+            self.persist_btn_icon_file_name, self.persist_btn_text, self
+        )
+        self.persist_btn.setStatusTip("Saves your current settings to the config file")
+        self.persist_btn.clicked.connect(self._save_config_to_disk)  # type: ignore
+        return (GridLayoutCell(self.persist_btn, span=4),)
 
     def _enable_checkbox_key_autodownload(self) -> None:
         self.checkbox_gpg_key_autodownload.setEnabled(
             len(self.field_keyserver_url.text().strip()) > 0
         )
 
+    def refresh_icon(self) -> None:
+        self.persist_btn.setIcon(Icon(self.persist_btn_icon_file_name))
+
 
 def group_box(
     name: str, *widgets: Sequence[Optional[Union[QtWidgets.QWidget, GridLayoutCell]]]
@@ -183,6 +196,48 @@ def group_box(
     return (GridLayoutCell(box, span=4),)
 
 
+def check_hostname(
+    regex: str, widget: LineEdit, config_key: str, status_tip: str
+) -> None:
+    """Verify that the URL entered in the specified widget is valid.
+    * If the URL is not valid, the cell is colored in red and an error is
+      displayed as tooltip.
+    * Minor errors (e.g. using :/ instead of :// after the scheme) are
+      auto-corrected.
+    """
+
+    def display_error_in_widget(error_msg: str) -> None:
+        # Change the background color and status tip of the LineEdit widget to
+        # display an error to the user.
+        widget.setStyleSheet(f"#{config_key} {{background-color:red;}}")
+        widget.setStatusTip(error_msg)
+
+    # Verify that a valid scheme is given by the user.
+    # If no scheme is given, https:// is added by default.
+    url = widget.text().strip()
+    scheme, *_ = re.split(r"://|:/", url)
+    if not _:
+        hostname = scheme
+        scheme = "https"
+    else:
+        hostname = _[0]
+        if scheme not in ("http", "https"):
+            display_error_in_widget(f"Non-allowed scheme: {scheme}://")
+            return
+
+    hostname_only, *_ = re.split(r":\d+|/", hostname)
+
+    # Verify that a the hostname has a valid syntax.
+    if not re.search(regex, hostname_only) and not re.search(REGEX_IP, hostname_only):
+        display_error_in_widget("Invalid hostname or IP address")
+        return
+
+    # Update widget text to the (possibly) corrected URL value.
+    widget.setText(scheme + "://" + hostname)
+    widget.setStyleSheet(f"#{config_key} {{}}")
+    widget.setStatusTip(status_tip)
+
+
 def widget_str(
     config_proxy: ConfigProxy,
     config_key: str,
@@ -193,9 +248,15 @@ def widget_str(
     widget.setStatusTip(status_tip)
     bind(config_proxy, config_key, widget, TextControl)
     if regex:
-        widget.setValidator(
-            QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(regex))
-        )
+        widget.setObjectName(config_key)
+        if regex == REGEX_FQDN:
+            widget.editingFinished.connect(  # type: ignore
+                lambda: check_hostname(regex, widget, config_key, status_tip)
+            )
+        else:
+            widget.setValidator(
+                QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(regex))
+            )
 
     return (widget,)
 
@@ -205,8 +266,11 @@ def widget_path(
     config_key: str,
     status_tip: str,
     file_type: FileType,
+    parent: IconRepainterWidget,
 ) -> Tuple[QtWidgets.QWidget, ...]:
-    widget = PathInput(directory=file_type is FileType.directory, path=None)
+    widget = PathInput(
+        directory=file_type is FileType.directory, path=None, parent=parent
+    )
     widget.setStatusTip(status_tip)
     bind(config_proxy, config_key, widget, PathControl)
 
@@ -275,6 +339,7 @@ def widget_row_from_field(
     config_proxy: ConfigProxy,
     field: "Field[Any]",
     widget_register: Optional[Dict[str, QtWidgets.QWidget]] = None,
+    parent: Optional[QtWidgets.QWidget] = None,
 ) -> Tuple[QtWidgets.QWidget, ...]:
     """Create a widget row consisting of a label, a main widget and possible
     auxiliary widgets based on field metadata of the Config dataclass.
@@ -292,16 +357,21 @@ def widget_row_from_field(
             "Field metadata is required in order to create a widget from a field."
         ) from None
     widget_factory = widget_by_type[field_type]
-    # Special cases:
-    if metadata.minimum and metadata.maximum:
-        widget_factory = widget_int_range
-    if metadata.file_type is not None:
-        widget_factory = widget_path
     widget_args = {
         key: val
         for key, val in asdict(metadata).items()
         if key not in ("label", "description") and val is not None
     }
+    # Special cases:
+    if (
+        metadata.minimum is not None
+        and metadata.maximum is not None
+        and abs(metadata.minimum - metadata.maximum) < 25
+    ):
+        widget_factory = widget_int_range
+    if metadata.file_type is not None:
+        widget_factory = widget_path
+        widget_args = {**widget_args, **{"parent": parent}}
     widget_row = widget_factory(
         config_proxy,
         config_key=field.name,
