@@ -4,7 +4,7 @@ from functools import partial
 from typing import Iterator, Optional, Sequence
 
 from .component import (
-    TabMixin,
+    BaseTab,
     PathInput,
     GuiProgress,
     get_text_input,
@@ -18,6 +18,8 @@ from .component import (
     hbox_layout,
 )
 from .file_selection_widget import ArchiveOnlyFileSelectionWidget
+from .theme import Action
+from .model import new_sftp_connection, AppData
 from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
 from .ui_model_bind import (
     bind,
@@ -26,7 +28,6 @@ from .ui_model_bind import (
     OptionalPasswordControl,
     OptionalPathControl,
 )
-from .model import new_sftp_connection, AppData
 from .. import protocols
 from ..core.error import UserError
 from ..protocols.liquid_files import Protocol as LiquidFiles
@@ -36,7 +37,7 @@ from ..utils.config import ConnectionStore
 from ..workflows import transfer
 
 
-class TransferTab(TabMixin):
+class TransferTab(BaseTab):
     """Class that creates the "Transfer" Tab of the GUI application."""
 
     def __init__(self, parent: QtWidgets.QMainWindow, app_data: AppData):
@@ -103,7 +104,7 @@ class TransferTab(TabMixin):
         text_jumphost.setStatusTip("(optional) URL of the jumphost server")
         bind(sftp_args, "jumphost", text_jumphost, OptionalTextControl)
 
-        pkey_location = PathInput(directory=False, path=None)
+        pkey_location = PathInput(directory=False, path=None, parent=self)
         pkey_location.setStatusTip(
             "Path to the private SSH key used for authentication"
         )
@@ -261,7 +262,7 @@ class TransferTab(TabMixin):
             ("Delete the current connection profile", delete, True, "trash-2"),
             ("Save the current connection profile", save, True, "save"),
         ):
-            action = QAction(QtGui.QIcon(f":icon/feather/{icon}.png"), tip, self)
+            action = Action(f":icon/feather/{icon}.png", tip, self)
             action.triggered.connect(fn)  # type: ignore
             if needs_selection:
                 connections_selection.currentTextChanged.connect(  # type: ignore
