@@ -10,7 +10,7 @@ from typing import Dict, Optional, Callable, Union, Any
 from libbiomedit.lib import deserialize as serialization
 
 from .log import get_default_log_dir, create_logger
-from .validation import REGEX_URL, PACKAGE_SUFFIX
+from .validation import REGEX_FQDN, PACKAGE_SUFFIX
 from .get_config_path import get_config_file, CONFIG_FILE_NAME
 from .. import APP_NAME_SHORT
 from ..core import gpg
@@ -118,13 +118,13 @@ class Config:
         label="DCC portal URL",
         description="URL of portal instance. The portal is used for key signing and "
         "DTR (Data Transfer Request) validation.",
-        regex=REGEX_URL,
+        regex=REGEX_FQDN,
     )
     keyserver_url: Optional[str] = field_ext(
-        default="keyserver.dcc.sib.swiss",
+        default="https://keyserver.dcc.sib.swiss",
         label="Keyserver URL",
         description="URL of the keyserver: used for publishing/fetching public PGP keys.",
-        # no validation, as specifying http:// is optional
+        regex=REGEX_FQDN,
     )
     gpg_home_dir: str = field_ext(
         serialize=reverse_expanduser_if_path_exists,
@@ -156,7 +156,7 @@ class Config:
         default="https://pypi.org",
         label="Repo URL",
         description="Python package repository, used when looking for updates.",
-        regex=REGEX_URL,
+        regex=REGEX_FQDN,
     )
     check_version: bool = field_ext(
         default=True,
@@ -255,6 +255,14 @@ class Config:
         description="Verify that the name of data packages follows the "
         f"{APP_NAME_SHORT} naming convention",
     )
+    sftp_buffer_size: int = field_ext(
+        default=1_048_576,
+        minimum=65_536,
+        maximum=16_777_216,
+        label="SFTP buffer size",
+        description="SFTP buffer size in bytes. Larger values usually result in "
+        "faster transfer speed on stable networks.",
+    )
 
     def __post_init__(self) -> None:
         for url in ("dcc_portal_url", "repo_url"):
