@@ -241,11 +241,10 @@ def log_to_rotating_file(
         logger = logging.getLogger()
     # Create log directory if it does not exist yet.
     log_dir_path = Path(log_dir)
-    if not log_dir_path.is_dir():
-        try:
-            log_dir_path.mkdir(parents=True)
-        except PermissionError:
-            logger.error("unable to create directory %s.", log_dir_path.as_posix())
+    try:
+        log_dir_path.mkdir(parents=True, exist_ok=True)
+    except PermissionError:
+        logger.error("unable to create directory %s.", log_dir_path.as_posix())
 
     # Setup handler for logging info to file. The file logging format adds
     # the data and time at the start of each logged message.
