@@ -1,6 +1,5 @@
 import os
 
-from ..core.error import UserError
 from ..utils.config import create_config
 from ..utils.get_config_path import get_config_file
 from ..utils.log import create_logger, log_runtime_info
@@ -13,6 +12,7 @@ def create() -> None:
     """Creates a new default config file in the users config dir."""
     config_file = get_config_file()
     if os.path.isfile(config_file):
-        raise UserError(f"The config file already exists at '{config_file}'.")
-    create_config()
-    logger.info("Created config file at '%s'", config_file)
+        logger.info("The config file already exists at '%s'.", config_file)
+    else:
+        create_config()
+        logger.info("Created config file at '%s'", config_file)
