@@ -52,37 +52,42 @@ def decrypt(
     progress: Optional[ProgressInterface] = None,
 ) -> None:
     """Main function of the decrypt workflow. Decrypts and decompresses the
-    input archive files."""
+    input archive files.
+    """
 
-    logger.info(
-        "File(s) to decrypt: [%s], output_dir: %s%s",
-        ", ".join(files),
-        output_dir,
-        " (dry_run)" if dry_run else "",
-    )
+    with logger.log_task(
+        f"Input data verification{' (dry_run)' if dry_run else ''}..."
+    ):
 
-    with logger.log_task("Input data check"):
+        # Verify input data packages extension and content.
+        logger.info("Files to decrypt: %s", ", ".join(files))
+        logger.info("Output directory: %s", output_dir)
         for archive_path in files:
+            logger.info("Verifying: %s", archive_path)
             check_package(archive_path)
 
-    check_space(get_total_size(files), output_dir, force=False)
+        # Verify sufficient disk space is available.
+        check_space(get_total_size(files), output_dir, force=False)
 
     if dry_run:
         logger.info("Dry run completed successfully.")
         return
 
     for archive_file in files:
-        # Reset progress for each tar file
-        if progress is not None:
-            progress.update(0.0)
-        decrypt_archive(
-            archive_file,
-            passphrase=passphrase,
-            output_dir=output_dir,
-            config=config,
-            decrypt_only=decrypt_only,
-            progress=progress,
-        )
+        with logger.log_task(f"Processing file: {archive_file}..."):
+
+            # Reset progress bar for each archive file.
+            if progress is not None:
+                progress.update(0.0)
+
+            decrypt_archive(
+                archive_file,
+                passphrase=passphrase,
+                output_dir=output_dir,
+                config=config,
+                decrypt_only=decrypt_only,
+                progress=progress,
+            )
 
 
 def decrypt_archive(
@@ -95,7 +100,6 @@ def decrypt_archive(
 ) -> None:
     """Decrypts and decompresses the input archive_file."""
 
-    logger.info("Processing file %s", archive_file)
     # To avoid overwriting files, each archive is unpacked in a directory
     # that has the same name as the archive file minus its extension.
     out_dir = unique_filename(
@@ -103,19 +107,18 @@ def decrypt_archive(
     )
 
     with DeleteDirOnError(out_dir):
-        with logger.log_task("Verifying signatures..."):
-            verify_metadata_signature(
-                tar_file=archive_file,
-                gpg_store=config.gpg_store,
-                signee_fingerprint=None,
-                key_authority_fingerprint=config.key_authority_fingerprint,
-                keyserver_url=config.keyserver_url,
-                validate_key_origin=False,
-            )
+        logger.info("Verifying signatures")
+        verify_metadata_signature(
+            tar_file=archive_file,
+            gpg_store=config.gpg_store,
+            signee_fingerprint=None,
+            key_authority_fingerprint=config.key_authority_fingerprint,
+            keyserver_url=config.keyserver_url,
+            validate_key_origin=False,
+        )
 
-        with logger.log_task("Verifying encryption keys..."), extract(
-            archive_file, DATA_FILE_ENCRYPTED
-        ) as f_data:
+        logger.info("Verifying encryption keys")
+        with extract(archive_file, DATA_FILE_ENCRYPTED) as f_data:
             keys = list(
                 retrieve_refresh_and_validate_keys(
                     key_search_terms=gpg.extract_key_id(f_data),
@@ -135,29 +138,23 @@ def decrypt_archive(
                 gpg_store=config.gpg_store,
             )
 
+        # Decrypt and decompress input files.
+        logger.info("Decrypting data")
         unpacked_files: List[str] = []
-        with logger.log_task("Decrypting data..."), subprogress(
+        with subprogress(
             progress, step_completion_increase=0.95
         ) as scaled_progress, extract_with_progress(
             archive_file, scaled_progress, DATA_FILE_ENCRYPTED
         ) as f_data:
             os.makedirs(out_dir, exist_ok=True)
-            if decrypt_only:
-                sender_fprs = core_decrypt(
-                    source=f_data,
-                    output=os.path.join(out_dir, DATA_ARCHIVE),
-                    gpg_store=config.gpg_store,
-                    passphrase=passphrase,
-                )
-            else:
-                sender_fprs = core_decrypt(
-                    source=f_data,
-                    output=partial(
-                        unpack_from_stream, dest=out_dir, content=unpacked_files
-                    ),
-                    gpg_store=config.gpg_store,
-                    passphrase=passphrase,
-                )
+            sender_fprs = core_decrypt(
+                source=f_data,
+                output=os.path.join(out_dir, DATA_ARCHIVE)
+                if decrypt_only
+                else partial(unpack_from_stream, dest=out_dir, content=unpacked_files),
+                gpg_store=config.gpg_store,
+                passphrase=passphrase,
+            )
             sender_sig_keys = retrieve_refresh_and_validate_keys(
                 key_search_terms=map(fingerprint2keyid, sender_fprs),
                 gpg_store=config.gpg_store,
@@ -172,23 +169,24 @@ def decrypt_archive(
                 ),
             )
 
+        # Compute decrypted data size and checksum verification (if data was
+        # decompressed).
         with subprogress(progress, step_completion_increase=0.05) as scaled_progress:
             if decrypt_only:
                 decryption_stats = "size: " + to_human_readable_size(
                     os.path.getsize(archive_file)
                 )
             else:
-                with logger.log_task(
-                    "Checksum verification of uncompressed data..."
-                ), open(os.path.join(out_dir, CHECKSUM_FILE), "rb") as fout:
-                    # Path separator replacement is a fail-safe if paths are not POSIX.
+                logger.info("Checksum verification of uncompressed data")
+                with open(os.path.join(out_dir, CHECKSUM_FILE), "rb") as fout:
+                    # Path separator replacement is a fail-safe if paths are
+                    # not POSIX.
                     verify_checksums(
                         [
                             (c, os.path.join(out_dir, p.replace("\\", os.path.sep)))
                             for c, p in read_checksum_file(fout)
                         ],
-                        # `max_workers` accepts only None and positive integers. Make
-                        # sure that zero and negative values are converted into None.
+                        # `max_workers` accepts only None and positive integers.
                         max_workers=config.max_cpu if config.max_cpu > 0 else None,
                     )
                 log_files(unpacked_files)
@@ -200,7 +198,7 @@ def decrypt_archive(
             if scaled_progress is not None:
                 scaled_progress.update(1.0)
             logger.info(
-                "Completed data decryption: %s (%s)", archive_file, decryption_stats
+                "Completed data decryption: %s [%s]", archive_file, decryption_stats
             )
 
 
