@@ -1,9 +1,9 @@
-from datetime import datetime
 import io
 import json
 import os
-from functools import partial
 from pathlib import Path
+from datetime import datetime
+from functools import partial
 from typing import List, Optional, Callable, Tuple, Iterable, Any, TypeVar
 from zipfile import ZipFile, ZipInfo, ZIP_STORED
 
@@ -39,17 +39,17 @@ from ..core.checksum import generate_checksums_file_content, compute_checksum_on
 from ..core.error import UserError
 from ..core.metadata import MetaData, alnum_str, Purpose, HexStr1024, HexStr256
 from ..core.secret import Secret
-from ..core.portal_api import PortalApi
 from ..utils.progress import ProgressInterface, subprogress, progress_file_iter
 from ..utils.config import Config
 from ..utils.log import create_logger, log_runtime_info, log_timing
 
 DATE_FMT_FILENAME = "%Y%m%dT%H%M%S"
 logger = create_logger(__name__)
-output_name_str = alnum_str(min_length=1, max_length=50, allow_dots=True)
 
 
-def path_str(directory: bool = False, writable: bool = False) -> Callable[[str], str]:
+def check_path(
+    directory: bool = False, writable: bool = False
+) -> Callable[[str], None]:
     """Generate a 'type definition' function that will check that a string is
     a valid path (file or directory).
 
@@ -59,7 +59,7 @@ def path_str(directory: bool = False, writable: bool = False) -> Callable[[str],
     :raises ValueError:
     """
 
-    def _path_str(path_to_check: str) -> str:
+    def _check_path(path_to_check: str) -> None:
         path = Path(path_to_check)
         if not path.exists():
             raise ValueError(f"Invalid path: '{path_to_check}'. Path does not exist.")
@@ -69,23 +69,20 @@ def path_str(directory: bool = False, writable: bool = False) -> Callable[[str],
             )
         if writable and not os.access(path.as_posix(), os.W_OK):
             raise ValueError(f"Invalid path: '{path_to_check}'. Path is not writable.")
-        return path_to_check
 
-    return _path_str
-
-
-output_path_str = path_str(directory=True, writable=True)
+    return _check_path
 
 
 def check_integer_in_range(min_value: int, max_value: int) -> Callable[[Any], None]:
-    """Generate a function that will check that its input value is an integer,
-    and that this integer is in the range [min_value:max_value].
+    """Generate a function which checks that:
+    * Its input value is an integer.
+    * Its input value is in the range [min_value:max_value].
     """
 
     def _integer_in_range(value_to_check: Any) -> None:
         try:
             value_to_check = int(value_to_check)
-        except ValueError as e:
+        except (ValueError, TypeError) as e:
             raise ValueError("Value must be an integer") from e
 
         if value_to_check < min_value or value_to_check > max_value:
@@ -95,10 +92,8 @@ def check_integer_in_range(min_value: int, max_value: int) -> Callable[[Any], No
 
 
 def check_paths_on_posix(paths: Iterable[str]) -> None:
-    """Check paths for windows-style separators.
-
-    If the machine is POSIX, verify that the specified paths do not contain
-    any windows-style separators.
+    """If the machine is POSIX, verify that the specified input path(s) do not
+    contain any windows-style separators (i.e. \\).
     """
 
     for path in paths:
@@ -135,48 +130,47 @@ def encrypt(  # pylint: disable=too-many-statements
 
     Main function of the encryption workflow. It compresses the input files
     into a single archive file, which is then encrypted for the specified
-    recipient and signed by the specified sender.
+    recipients and signed by the specified sender.
+
     Finally, the encrypted data is bundled with a metadata file in a single
     .zip archive.
-    The function returns the file name of the created package.
+
+    The function returns the file name of the created data package.
     """
 
-    # Retrive non-specified, optional argument values from config.
-    if sender is None:
-        sender = config.default_sender or config.gpg_store.default_key()
-        if sender is None:
-            raise UserError("Sender not specified with no default sender.")
-    if compression_level is None:
-        compression_level = config.compression_level
-    check_arg_value(
-        arg_value=compression_level,
-        arg_name="compression level",
-        arg_type_checker=check_compression_level,
-    )
+    with logger.log_task(
+        f"Input data verification{' (dry_run)' if dry_run else ''}..."
+    ):
 
-    if not files:
-        raise UserError("Empty file list")
-
-    files_to_encrypt = list(search_files_recursively(files))
-
-    with logger.log_task("Input data check"):
+        # Verify user has specified at least 1 file to encrypt and that the
+        # file(s) have read permission for the user.
+        logger.info("Verify files to encrypt")
+        if not files:
+            raise UserError("Empty file list.")
+        files_to_encrypt = list(search_files_recursively(files))
         if not files_to_encrypt:
             raise UserError(
                 "No input files found. Did you try encrypting an empty directory?"
             )
         check_file_read_permission(files_to_encrypt)
-        # Retrieve the lowest common directory of all input files/directories.
-        root_dir = os.path.commonpath(
-            [Path(x).absolute().parent.as_posix() for x in files]
-        )
-        if dtr_id is None and verify_dtr:
-            raise UserError("DTR (Data Transfer Request) ID is missing.")
-        if output_suffix:
-            check_arg_value(output_suffix, "output suffix", output_name_str)
 
-    with logger.log_task("Retrieve sender and recipient GnuPG keys"):
+        # If no sender is specified, retrieve default sender from config.
+        if sender is None:
+            sender = config.default_sender or config.gpg_store.default_key()
+            if sender is None:
+                raise UserError("Sender not specified with no default sender.")
+
+        # If no compression level is specified, retrieve it from config.
+        if compression_level is None:
+            compression_level = config.compression_level
+        check_arg_value(
+            arg_value=compression_level,
+            arg_name="compression level",
+            arg_type_checker=check_compression_level,
+        )
+
         # Retrieve the sender's public and private keys, as well as the
-        # recipient's public key. Here is what these keys are needed for:
+        # recipient's public key:
         #  - sender private key: needed to sign the encrypted data.
         #  - sender public key : will be checked to make sure it is signed by
         #       the DCC. Only private keys with a matching public key that is
@@ -185,6 +179,7 @@ def encrypt(  # pylint: disable=too-many-statements
         #
         # The sender/recipient information can be either an email, a keyID or
         # a full fingerprint.
+        logger.info("Retrieve sender and recipient GnuPG keys")
         sender_pub_key, *recipients_pub_key = retrieve_refresh_and_validate_keys(
             key_search_terms=(sender, *recipient),
             gpg_store=config.gpg_store,
@@ -198,66 +193,70 @@ def encrypt(  # pylint: disable=too-many-statements
         # itself is not needed because it shares the fingerprint with the
         # public key.
         search_priv_key(sender_pub_key.fingerprint, config.gpg_store)
-
         logger.info(
-            "Sender: %s. Recipients: %s",
+            "Sender: %s",
             f"{sender_pub_key.uids[0]} ({sender_pub_key.fingerprint})",
+        )
+        logger.info(
+            "Recipients: %s",
             ", ".join(
                 f"{key.uids[0]} ({key.fingerprint})" for key in recipients_pub_key
             ),
         )
 
-    project_code = determine_project_code(
-        dtr_id,
-        purpose,
-        config.portal_api if not config.offline and verify_dtr else None,
-        sender_pub_key,
-        recipients_pub_key,
-    )
-
-    timestamp = datetime.now().astimezone()
-    # The default value for the output name is based on date and time
-    # when the script is being run.
-    # Example output name is "20191011T145012".
-    output_name = generate_output_archive_name(
-        output,
-        default="_".join(
-            filter(
-                None,
-                [
-                    project_code,
-                    timestamp.strftime(DATE_FMT_FILENAME),
-                    output_suffix or config.package_name_suffix,
-                ],
+        # If requested, verify data transfer related information.
+        if verify_dtr:
+            logger.info("Verify Data Transfer ID")
+            # Connect to the portal API to retrieve the code (abbreviation)
+            # associated with the DTR ID.
+            project_code = verify_dtr_info_and_get_project_code(
+                dtr_id, purpose, config, sender_pub_key, recipients_pub_key
             )
-        ),
-    )
-    total_input_file_size = get_total_size(files_to_encrypt)
-    check_space(total_input_file_size, os.path.dirname(output_name), force=force)
+            logger.info("DTR ID '%s' is valid for project '%s'", dtr_id, project_code)
+        else:
+            project_code = None
 
-    # Create a list of file paths (of the files that are being packaged) as they
-    # will appear in the output archive file.
-    archive_paths = [
-        os.path.join(CONTENT_FOLDER, os.path.relpath(f, start=root_dir))
-        for f in files_to_encrypt
-    ]
+        logger.info("Verify available disk space")
+        # The default value for the output name is based on date and time
+        # when the script is being run.
+        # Example output name: "20191011T145012.zip".
+        timestamp = datetime.now().astimezone()
+        output_name = generate_output_archive_name(
+            prefix=project_code,
+            timestamp=timestamp,
+            suffix=output_suffix or config.package_name_suffix,
+            dir_or_name_override=output,
+        )
+        total_input_file_size = get_total_size(files_to_encrypt)
+        check_space(total_input_file_size, os.path.dirname(output_name), force=force)
 
-    check_paths_on_posix(archive_paths)
+        # Create a list of file paths (i.e. the files to package) as they will
+        # appear in the output archive file. For this we start by retrieving
+        # the lowest common directory of all input files/directories.
+        root_dir = os.path.commonpath(
+            [Path(x).absolute().parent.as_posix() for x in files]
+        )
+        archive_paths = [
+            os.path.join(CONTENT_FOLDER, os.path.relpath(f, start=root_dir))
+            for f in files_to_encrypt
+        ]
+        check_paths_on_posix(archive_paths)
+
+        # If the user asked to sign the data, check that the GPG key password
+        # is correct.
+        if config.sign_encrypted_data and not dry_run:
+            logger.info("Verify data sender GPG password")
+            check_password(
+                password=enforce_passphrase(passphrase),
+                key_fingerprint=sender_pub_key.fingerprint,
+                gpg_store=config.gpg_store,
+            )
 
     if dry_run:
         logger.info("Dry run completed successfully")
         return None
 
-    # If the user asked to sign the data, check that the GPG key password
-    # is correct.
-    if config.sign_encrypted_data:
-        check_password(
-            password=enforce_passphrase(passphrase),
-            key_fingerprint=sender_pub_key.fingerprint,
-            gpg_store=config.gpg_store,
-        )
-
-    with logger.log_task("Compute sha256 checksum on input files"):
+    with logger.log_task("Compute sha256 checksum on input files..."):
         # Write input file checksums to a file that will be added to the
         # encrypted .tar.gz archive. This information must be encrypted as
         # file names sometimes contain information about their content.
@@ -270,7 +269,7 @@ def encrypt(  # pylint: disable=too-many-statements
         if progress is not None:
             progress.update(0.1)
 
-    with logger.log_task("Compress and encrypt input data [this can take a while]"):
+    with logger.log_task("Compress and encrypt input data [this can take a while]..."):
         # Encryption is done with the recipient's public key and the optional
         # signing with the user's (i.e sender) private key. The user's private
         # key passphrase is needed to sign the encrypted file.
@@ -353,7 +352,7 @@ def encrypt(  # pylint: disable=too-many-statements
                 )
 
     logger.info(
-        "Completed data encryption: %s (%s)",
+        "Completed data encryption: %s [%s]",
         output_name,
         get_compression_stats(total_input_file_size, os.path.getsize(output_name)),
     )
@@ -369,7 +368,7 @@ def check_arg_value(
     arg_type_checker: Callable[[T], Any],
 ) -> None:
     """Verify that the value of variable arg_value that is named arg_name is
-    following type arg_type. arg_type is a function that does a check
+    following type arg_type. arg_type_checker is a function that does a check
     of the type of the variable.
     """
     try:
@@ -384,6 +383,12 @@ def byte_encode_metadata(
     passphrase: Optional[Secret[str]],
     sender_pub_key: Optional[gpg.Key],
 ) -> Tuple[bytes, bytes]:
+    """Encodes the provided metadata and creates a detached PGP signature file
+    for the metadata if a PGP key (sender_pub_key) is specified.
+
+    The function returns both the metadata and associated detached signature in
+    encoded form.
+    """
     metadata_bytes = json.dumps(MetaData.asdict(metadata), indent=4).encode()
     metadata_signature_bytes = b""
     if sender_pub_key is not None:
@@ -396,21 +401,36 @@ def byte_encode_metadata(
     return metadata_bytes, metadata_signature_bytes
 
 
-def determine_project_code(
+def verify_dtr_info_and_get_project_code(
     dtr_id: Optional[int],
     purpose: Optional[Purpose],
-    portal_api: Optional[PortalApi],
+    config: Config,
     sender_pub_key: gpg.Key,
     recipients_pub_key: Iterable[gpg.Key],
-) -> Optional[str]:
+) -> str:
+    """Determine the "code" (i.e. short name) of a project based on its DTR ID
+    (Data Transfer ID) and perform some sanity checks on data associated with
+    the tranfer by querying the portal API:
+     * Verify the data sender's key (sender_pub_key) is authorized.
+     * Verify the data recipients' keys (recipients_pub_key) is authorized and
+       that the recipient is a DM (data manager) for the project.
+     * Verify that the purpose is correct.
+
+    If all these checks complete successfully, the project "code" is returned.
+    """
+
+    error_prefix = "Cannot verify DTR (Data Transfer Request) ID"
     if dtr_id is None:
-        return None
-    if purpose is None:
-        raise UserError("DTR ID specified but `purpose` is missing")
-    if portal_api is None:
-        return None
+        raise UserError(f"{error_prefix}: 'DTR ID' is missing.")
+    if not purpose:
+        raise UserError(f"{error_prefix}: 'purpose' is missing.")
+    if config.offline:
+        raise UserError(f"{error_prefix}: offline mode is on.")
+    if not config.portal_api:
+        raise UserError(f"{error_prefix}: no portal URL specified in config.")
+
     try:
-        project_code = portal_api.verify_transfer(
+        project_code = config.portal_api.verify_transfer(
             metadata=MetaData(
                 transfer_id=dtr_id,
                 sender=HexStr1024(sender_pub_key.fingerprint),
@@ -420,56 +440,75 @@ def determine_project_code(
             ),
             filename="missing",
         )
-        logger.info("DTR ID '%s' is valid for project '%s'", dtr_id, project_code)
     except RuntimeError as e:
         raise UserError(format(e)) from e
     return project_code
 
 
-def generate_output_archive_name(output_name: Optional[str], default: str) -> str:
+def generate_output_archive_name(
+    prefix: Optional[str] = None,
+    timestamp: Optional[datetime] = None,
+    suffix: Optional[str] = None,
+    dir_or_name_override: Optional[str] = None,
+) -> str:
     """Generates the path + name of the output archive file of the encrypt
-    workflow.
-    If output_name does not contain any path information, the output directory
-    is set to the current working directory.
+    workflow and verify the user has write access to the output directory.
 
-    :param output_name: name or path + name of output tarball.
-    :param default: default file name to use if output_name is None or
-        output_name is a folder
-    :return: path and name of the output tarball file.
+    By default, the generated output name has the following structure:
+
+        <current work dir>/<prefix>_<date_and_time>_<suffix>.zip
+
+    Where <prefix> and <suffix> are optional and omitted if absent.
+    If an override value is passed (dir_or_name_override), it is used as
+    output name and/or output directory.
+
+    :param prefix: optional prefix for the output archive file.
+    :param timestamp: date and time to be used in archive file name (if no override
+        value is passed). If no value is provided, the current time is used.
+    :param suffix: optional suffix for the output archive file.
+    :param dir_or_name_override: directory, name or directory + name to use
+        for the output archive. If this value is None, a default output name
+        is generated as explained above.
+    :return: path and name of the output archive file.
     :raises UserError:
     """
-    if output_name is None:
-        output_name = default
-    else:
-        check_arg_value(Path(output_name).name, "output name", output_name_str)
-        check_arg_value(
-            Path(output_name).as_posix()
-            if Path(output_name).is_dir()
-            else Path(output_name).parent.as_posix(),
-            "path in output name",
-            output_path_str,
-        )
 
-    if Path(output_name).is_dir():
-        output_name = os.path.join(output_name, default)
+    # If no timestamp was provided, use the current time instead. Note that
+    # we might need this even if dir_or_name_override is provided since at
+    # this point we don't know whether the latter is a directory or file name.
+    if not timestamp:
+        timestamp = datetime.now().astimezone()
+    default_name = "_".join(
+        filter(None, [prefix, timestamp.strftime(DATE_FMT_FILENAME), suffix])
+    )
+
+    if dir_or_name_override is None:
+        output_name = default_name
+        output_dir = Path.cwd().as_posix()
+    else:
+        override_path = Path(dir_or_name_override)
+        if override_path.is_dir():
+            output_name = default_name
+            output_dir = os.path.realpath(override_path.as_posix())
+        else:
+            output_name = override_path.name
+            output_dir = os.path.realpath(override_path.parent.as_posix())
+
     # Add '.zip' extension to output name if needed.
     if not output_name.endswith(".zip"):
-        output_name = output_name + ".zip"
+        output_name += ".zip"
 
-    # If output_name does not contain any path info, the output path is
-    # set to the current working directory.
-    basename = Path(output_name).name
-    if basename == output_name:
-        output_dir = Path.cwd()
-    else:
-        output_dir = Path(output_name).parent
+    # Verify that output name and path follow the conventions. This also
+    # verifies that the specified output directory exists and that the user
+    # has write access to it.
+    check_arg_value(
+        output_name,
+        "output name",
+        alnum_str(min_length=1, max_length=60, allow_dots=True),
+    )
+    check_arg_value(
+        output_dir, "path in output name", check_path(directory=True, writable=True)
+    )
 
-    # Verify that user has write permission to output directory. We do this
-    # check now because the procedure of compression + encryption can take
-    # a long time and we want to be able to warn the user immediately.
-    if not output_dir.is_dir():
-        raise UserError(f"output directory does not exist: {output_dir}")
-    if not os.access(output_dir.as_posix(), os.W_OK):
-        raise UserError(f"no write permission on directory: {output_dir}")
-
-    return output_dir.joinpath(basename).as_posix()
+    # Return output archive name in posix format.
+    return (Path(output_dir) / output_name).as_posix()
