@@ -10,6 +10,17 @@ from ..utils.config import Config
 logger = create_logger(__name__)
 
 
+@log_runtime_info(logger)
+def verify_keylengths_and_request_sigs(
+    key_ids: Sequence[str], *, config: Config
+) -> None:
+    """Verify key lengths and send sign request"""
+    keys = frozenset(search_pub_key(k, config.gpg_store, sigs=False) for k in key_ids)
+    for key in keys:
+        verify_key_length(key)
+    request_sigs(key_ids=key_ids, config=config)
+
+
 @log_runtime_info(logger)
 def request_sigs(key_ids: Sequence[str], *, config: Config) -> None:
     """Sends a request to sign the specified keys to the key validation
@@ -19,8 +30,6 @@ def request_sigs(key_ids: Sequence[str], *, config: Config) -> None:
     if config.offline:
         raise UserError("Requesting key signature is not possible in offline mode.")
     keys = frozenset(search_pub_key(k, config.gpg_store, sigs=False) for k in key_ids)
-    for key in keys:
-        verify_key_length(key)
     for key in keys:
         upload_keys((key.fingerprint,), config=config)
         logger.info("Sending a request for '%s'", key.key_id)
