@@ -143,16 +143,15 @@ def transfer(
         for recipient_keys, r_files in files_by_recipient.items():
             emails = [get_recipient_email(k) for k in recipient_keys]
 
-            # LiquidFiles protocol: email addresses of the recipient must be
-            # specified as part of the protocol.
-            if hasattr(protocol, "recipients"):
-                setattr(protocol, "recipients", emails)
-
-            # SFTP protocol: encoding of public SSH key must be specified.
-            if hasattr(protocol, "pkey_password_encoding"):
-                setattr(
-                    protocol, "pkey_password_encoding", config.ssh_password_encoding
-                )
+            for label, value in (
+                # LiquidFiles: email addresses of recipient must be specified.
+                ("recipients", emails),
+                # SFTP: encoding of public SSH key and buffer size must be specified.
+                ("pkey_password_encoding", config.ssh_password_encoding),
+                ("buffer_size", config.sftp_buffer_size),
+            ):
+                if hasattr(protocol, label):
+                    setattr(protocol, label, value)
 
             logger.info("Transferring files encrypted for: %s", ", ".join(emails))
             protocol.upload(
